unit FreeOTFEfmeOptions_PKCS11;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, FreeOTFEfmeOptions_Base, StdCtrls, 
  OTFEFreeOTFE_U, FreeOTFESettings, SDUFrames, SDUFilenameEdit_U,
  OTFEFreeOTFE_VolumeSelect;

type
  TfmeOptions_PKCS11 = class(TfmeOptions_Base)
    gbPKCS11: TGroupBox;
    lblLibrary: TLabel;
    ckEnablePKCS11: TCheckBox;
    pbVerify: TButton;
    gbPKCS11AutoActions: TGroupBox;
    lblAutoMountVolume: TLabel;
    ckPKCS11AutoDismount: TCheckBox;
    ckPKCS11AutoMount: TCheckBox;
    feLibFilename: TSDUFilenameEdit;
    OTFEFreeOTFEVolumeSelect1: TOTFEFreeOTFEVolumeSelect;
    procedure pbVerifyClick(Sender: TObject);
    procedure ControlChanged(Sender: TObject);
  private
    function LibraryDLL(): string;
    function VerifyLibrary(): boolean;

  public
    OTFEFreeOTFE: TOTFEFreeOTFE;

    procedure EnableDisableControls(); override;
    procedure Initialize(); override;
    procedure ReadSettings(config: TFreeOTFESettings); override;
    function  CheckSettings(): boolean; override;
    procedure WriteSettings(config: TFreeOTFESettings); override;
  end;

implementation

{$R *.dfm}

uses
  SDUi18n,
  SDUGeneral,
  SDUDialogs,
  pkcs11_library;

resourcestring
  FILTER_LIBRARY_FILES = 'Library files (*.dll)|*.dll|All files|*.*';

procedure TfmeOptions_PKCS11.ControlChanged(Sender: TObject);
begin
  inherited;
  EnableDisableControls();

end;

procedure TfmeOptions_PKCS11.pbVerifyClick(Sender: TObject);
begin
  inherited;
  
  if VerifyLibrary() then
    begin
    SDUMessageDlg(_('PKCS#11 library appears to be functional'), mtInformation);
    end

end;

procedure TfmeOptions_PKCS11.Initialize();
begin
  SDUCenterControl(gbPKCS11, ccHorizontal);
  SDUCenterControl(gbPKCS11, ccVertical, 25);

  feLibFilename.DefaultExt := 'dll';
  feLibFilename.OpenDialog.Options := feLibFilename.OpenDialog.Options + [ofPathMustExist, ofFileMustExist, ofForceShowHidden];
  feLibFilename.Filter := FILTER_LIBRARY_FILES;
  feLibFilename.FilterIndex := 0;

  OTFEFreeOTFEVolumeSelect1.OTFEFreeOTFE := OTFEFreeOTFE;
  OTFEFreeOTFEVolumeSelect1.FileSelectFilter     := FILE_FILTER_FLT_VOLUMES;
  OTFEFreeOTFEVolumeSelect1.FileSelectDefaultExt := FILE_FILTER_DFLT_VOLUMES;

end;

procedure TfmeOptions_PKCS11.ReadSettings(config: TFreeOTFESettings);
begin
  // General...
  ckEnablePKCS11.checked := config.OptPKCS11Enable;
  feLibFilename.Filename := config.OptPKCS11Library;

  ckPKCS11AutoMount.checked := config.OptPKCS11AutoMount;
  OTFEFreeOTFEVolumeSelect1.Filename := config.OptPKCS11AutoMountVolume;
  ckPKCS11AutoDismount.checked := config.OptPKCS11AutoDismount;

end;

procedure TfmeOptions_PKCS11.WriteSettings(config: TFreeOTFESettings);
begin
  // General...
  config.OptPKCS11Enable := ckEnablePKCS11.checked;
  config.OptPKCS11Library := LibraryDLL;
  config.OptPKCS11AutoMount := ckPKCS11AutoMount.checked;
  config.OptPKCS11AutoMountVolume := OTFEFreeOTFEVolumeSelect1.Filename;
  config.OptPKCS11AutoDismount := ckPKCS11AutoDismount.checked;

end;

function TfmeOptions_PKCS11.LibraryDLL(): string;
begin
  Result := trim(feLibFilename.Filename);
end;

function TfmeOptions_PKCS11.VerifyLibrary(): boolean;
var
  lib: TPKCS11Library;
  allOK: boolean;
begin
  allOK := TRUE;
  if (LibraryDLL = '') then
    begin
    SDUMessageDlg(_('Please specify the PKCS#11 library to be used'), mtError);
    allOK := FALSE;
    end;

  if allOK then
    begin
    allOK:= FALSE;
    try
      lib:= TPKCS11Library.Create(LibraryDLL);
      lib.Initialize();
      lib.Finalize();
      lib.Free();

      allOK := TRUE;
    except
      on E:Exception do
        begin
        // Nothing - just swallow it
        end;
    end;

    if not(allOK) then
      begin
      SDUMessageDlg(_('The library specified does not appear to be a valid/working PKCS#11 library'), mtError);
      end;
    end;

  Result := allOK;
end;


function TfmeOptions_PKCS11.CheckSettings(): boolean;
var
  allOK: boolean;
begin
  allOK := inherited CheckSettings();

  if allOK then
    begin
    if ckEnablePKCS11.checked then
      begin
      allOK := VerifyLibrary();
      end;
    end;

  if ckEnablePKCS11.checked then
    begin
    if allOK then
      begin
      if ckPKCS11AutoMount.checked then
        begin
        if (OTFEFreeOTFEVolumeSelect1.Filename = '') then
          begin
          SDUMessageDlg(
                        _('If automount on PKCS#11 token insertion is enabled, the volume to be mounted must be specified'),
                        mtError
                       );
          allOK := FALSE;
          end;
        end;

      end;
    end;

  Result := allOK;
end;

procedure TfmeOptions_PKCS11.EnableDisableControls();
var
  pkcs11Enabled: boolean;
begin
  inherited;

  pkcs11Enabled := ckEnablePKCS11.checked;

  SDUEnableControl(feLibFilename, pkcs11Enabled);
  SDUEnableControl(lblLibrary,    pkcs11Enabled);
  SDUEnableControl(pbVerify,      (
                                   pkcs11Enabled and
                                   (LibraryDLL <> '')
                                  ));


  SDUEnableControl(gbPKCS11AutoActions, pkcs11Enabled);
  SDUEnableControl(OTFEFreeOTFEVolumeSelect1, (
                                               pkcs11Enabled and  // Yes, this is needed; otherwise it'll enable/disable regardless
                                               ckPKCS11AutoMount.checked
                                              ));
  SDUEnableControl(lblAutoMountVolume, OTFEFreeOTFEVolumeSelect1.enabled);

end;

END.

